/********************************************************************
*--------------------------------------------------------------------
*    Titolo        :  C Enhanced - es16C
                      LED comandati da PWM steering con duty cycle
*          			  variabile in passi di 40ms.
*                     Impiego del modulo ECCP/PWM1.
*                     L'uscita dei PWM  il default
*    Data          :  01-05-2011
*    Modificato il :
*    Versione      :  V0.0
*    Ref. Hardware :  16F1829
*    Autore        :  afg
*
*--------------------------------------------------------------------
*********************************************************************
*  Impiego pin :
*  ----------------
*       16F1829 @ 20 pin
*
*                  |\/|
*             Vdd -|1   20|- Vss
*       RA5/CLKIN -|2   19|- RA0/ICSPDAT
*      RA4/CLKOUT -|3   18|- RA1/ICSPCLK
*        RA3/MCLR -|4   17|- RA2
*             RC5 -|5   16|- RCO
*             RC4 -|6   15|- RC1
*             RC3 -|7   14|- RC2
*             RC6 -|8   13|- RB4
*             RC7 -|9   12|- RB5
*             RB7 -|10  11|- RB6
*                  |______|
*
* Impiego pin:
*    Vdd      1: ++
*    RA5      2: In
*    RA4      3: In  
*    RA3      4: In  
*    RC5      5: Out P1A
*    RC4      6: Out P1B
*    RC3      7: Out P1C
*    RC6      8: In
*    RC7      9: In  
*    RB7     10: In  
*    RB6     11: In  
*    RB5     12: In  
*    RB4     13: In  
*    RC2     14: Out P1D
*    RC1     15: In
*    RC0     16: In
*    RA2     17: In
*    RA1     18: In  
*    RA0     19: In  
*    Vss     20: --
*
* Note: - INTOSC default @ 500kHz*
********************************************************************/

// intosc, no clockout, no wdt, no pwrt, no mclr, bor, no code prot.
// no WRT, no pll, no stack error, bor low, no bor lp, no LVP
#include "C:/Corso_C/MyIncludes/conf1829.h"

#include <xc.h>
#include <stdint.h>

#define _XTAL_FREQ   4000000         // OSCINT 4MHz 
#define isrlimit = 40;               // per 40ms
 
// 	FUNZIONI.
// clock a 4MHz
void oscint_4M(void){
    OSCCONbits.IRCF = 0b1101;       // intosc 4MHz
}

// inizializza I/O
void initialize_io(void){
    ANSELA = 0;                 // no analogiche
    ANSELB = 0;                  
    ANSELC = 0;                      
    TRISC = 0b11000011;         // RC5:2 out
    // abilita wpu per ingressi
    nWPUEN = 0;                 //OPTION_REGbits.nWPUEN = 0	
}

// setup Timer2 per 1kHz, clock Fosc/4
void initialize_tmr2(void){    
    //T2CLKCON = 0x00;           // T2CS FOSC/4; 
    //T2HLT = 0x00;
    //T2RST = 0x00;    
    TMR2IF = 0;
    T2CONbits.T2CKPS = 0b01;    // prescaler 1:8 postscaler 1:1         
	PR2 = 250-1;
}
// Timer2 on & clear flag
void tmr2_on(void){
    TMR2IF = 0;
    TMR2ON = 1;
}

 // setup ECCP/PWM1
void initialize_epwm1(void){
    //CCPTMRSbits.P1TSEL = 0;      // Select Timer2
    CCP1CONbits.P1M = 0b00;      // modo singolo    
	CCP1CONbits.CCP1M =0b1101;   // PWM mode: PxA/PxC active-high PxB/PxD active-low 
    CCP1CONbits.DC1B = 0b00;     // LSb duty = 0 
	CCPR1L = 125;                // MSb 50%
    //CCP1AS = 0;
    //PWM1CON = 0;
    PSTR1CON = 0b00011111;       // steering    
}

// abilita interruzioni
 void enable_irq(void){
    TMR2IF = 0;
    TMR2IE = 1;
	PEIE = 1;
	GIE = 1;
}	
 
/************************* MAIN PROGRAM ****************************/
void main(){
    
   oscint_4M();
   initialize_io();            // Inizializza I/O
   initialize_tmr2();          // setup Timer
   initialize_epwm1();         // setup ECCP 
   tmr2_on();                  // timer on
   enable_irq();               // abilita interruzioni   
   
// Loop principale 
  while (1){};
  }
	

/**** Interrupt Routine ****/
void interrupt isr(void){           // C90
//void __interrupt() isr(void){     // C99
static unsigned char  isrcntr = 0;   // contatore passi
    TMR2IF = 0;                 // cancella flag
	isrcntr++;
	if (isrcntr==40){
	  isrcntr = 0;
	  CCPR1L += 1;              // modifica duty cycle
	}  
}

