/********************************************************************
*--------------------------------------------------------------------
*    Titolo        :  C Enhanced - es16C_1
                      Buzzer piezo comandato da PWM steering con duty
*           		  cycle fisso.
*                     Impiego del modulo ECCP/PWM2.
*                     Uscita dei PWM rilocata con APFCON1.
*    Data          :  01-05-2011
*    Modificato il :
*    Versione      :  V0.0
*    Ref. Hardware :  16F1829
*    Autore        :  afg
*
*--------------------------------------------------------------------
*********************************************************************
*  Impiego pin :
*  ----------------
*       16F1829 @ 20 pin
*
*                  |\/|
*             Vdd -|1   20|- Vss
*       RA5/CLKIN -|2   19|- RA0/ICSPDAT
*      RA4/CLKOUT -|3   18|- RA1/ICSPCLK
*        RA3/MCLR -|4   17|- RA2
*             RC5 -|5   16|- RCO
*             RC4 -|6   15|- RC1
*             RC3 -|7   14|- RC2
*             RC6 -|8   13|- RB4
*             RC7 -|9   12|- RB5
*             RB7 -|10  11|- RB6
*                  |______|
*
* Impiego pin:
*    Vdd      1: ++
*    RA5      2: In
*    RA4      3: In
*    RA3      4: MCLR
*    RC5      5: In
*    RC4      6: In
*    RC3      7: Out P2A
*    RC6      8: In
*    RC7      9: In  
*    RB7     10: In  PB1
*    RB6     11: In  
*    RB5     12: In  
*    RB4     13: In  
*    RC2     14: Out P2B
*    RC1     15: In
*    RC0     16: In
*    RA2     17: Out LED
*    RA1     18: In  
*    RA0     19: In  
*    Vss     20: --
*
* Note: - INTOSC default @ 500kHz*
********************************************************************/

// intosc, no clockout, no wdt, no pwrt, mclr, bor, no code prot.
// no WRT, no pll, no stack error, bor low, no bor lp, no LVP
#include "C:/Corso_C/MyIncludes/conf1829.h"

#include <xc.h>
#include <stdint.h>

#define _XTAL_FREQ   4000000         // OSCINT 4MHz 
 
#define buttonled  LATAbits.LATA2          // LED di segnalazione
#define PB1        PORTBbits.RB7         // pulsante
 
// 	FUNZIONI.
// clock a 4MHz
void oscint_4M(void){
    OSCCONbits.IRCF = 0b1101;       // intosc 4MHz
}

// inizializza I/O
void initialize_io(void){
    ANSELA = 0;                 // no analogiche
    ANSELB = 0;                  
    ANSELC = 0;            
    LATA = 0;                  // preset led spento
    TRISA = 0b00111011;        // RA2 out
 // abilita wpu per ingressi
    nWPUEN = 0;                 //OPTION_REGbits.nWPUEN = 0	
	// rilocare P2A su RA5  P2B su RA4
	//APFCON1bits.CCP2SEL = 1;
	//APFCON1bits.P2BSEL = 1;
}

// setup Timer2 per 4kHz, clock Fosc/4
void initialize_tmr2(void){    
    //T2CLKCON = 0x00;           // T2CS FOSC/4; 
    T2CONbits.T2CKPS = 0b00;    // prescaler 1:1 postscaler 1:1         
	PR2 = 250-1;
}
// Timer2 on & clear flag
void tmr2_on(void){
    TMR2IF = 0;
    TMR2ON = 1;
}

 // setup ECCP/PWM2
void initialize_epwm2(void){
    //CCPTMRSbits.P2TSEL = 0;      // Select Timer2
    CCP2CONbits.P2M = 0b00;      // modo singolo    
	CCP2CONbits.CCP2M =0b1101;   // PWM mode: PxA active-high PxB active-low 
    CCPR2L = 125;         // duty 50%
    CCP2CONbits.DC2B = 0b00;     
    //CCP1AS = 0;
    //PWM1CON = 0;
    PSTR2CON = 0b00011111;       // steering    
}

 
/************************* MAIN PROGRAM ****************************/
void main(){
    
   oscint_4M();
   initialize_io();            // Inizializza I/O
   initialize_tmr2();          // setup Timer
   initialize_epwm2();         // setup ECCP 
   tmr2_on();                  // timer on
   
// Loop principale 
  while (1){

    while (PB1);              // attesa pulsante premuto
	TRISC = 0b11110011;       // uscita PWM
	buttonled = 1;            // LED acceso
	__delay_ms (200);         // delay minimo 
    
	while (!PB1);             // attesa rilascio
	TRISC = 0xFF;             // stop uscita PWM
	buttonled = 0;            // LED spento
    __delay_ms(30);	          // debounce
  }
}   


