/********************************************************************
*--------------------------------------------------------------------
*    Titolo        :  C Enhanced - es16C_2
*                     Half bridge per LED bicolore.
*                     Variazione colore automatica.
*                     Modulo ECCP/PWM2.
*    Data          :  01-05-2011
*    Modificato il :
*    Versione      :  V0.0
*    Ref. Hardware :  16F1829
*    Autore        :  afg
*
*--------------------------------------------------------------------
*********************************************************************
*  Impiego pin :
*  ----------------
*       16F1829 @ 20 pin
*
*                  |\/|
*             Vdd -|1   20|- Vss
*       RA5/CLKIN -|2   19|- RA0/ICSPDAT
*      RA4/CLKOUT -|3   18|- RA1/ICSPCLK
*        RA3/MCLR -|4   17|- RA2
*             RC5 -|5   16|- RCO
*             RC4 -|6   15|- RC1
*             RC3 -|7   14|- RC2
*             RC6 -|8   13|- RB4
*             RC7 -|9   12|- RB5
*             RB7 -|10  11|- RB6
*                  |______|
*
* Impiego pin:
*    Vdd      1: ++
*    RA5      2: In
*    RA4      3: In
*    RA3      4: MCLR
*    RC5      5: In
*    RC4      6: In
*    RC3      7: Out P2A
*    RC6      8: In
*    RC7      9: In  
*    RB7     10: In
*    RB6     11: In  
*    RB5     12: In  
*    RB4     13: In  
*    RC2     14: Out P2B
*    RC1     15: In
*    RC0     16: In
*    RA2     17: In
*    RA1     18: In  
*    RA0     19: In  
*    Vss     20: --
*
* Note: - INTOSC default @ 500kHz*
********************************************************************/

// intosc, no clockout, no wdt, no pwrt, mclr, bor, no code prot.
// no WRT, no pll, no stack error, bor low, no bor lp, no LVP
#include "C:/Corso_C/MyIncludes/conf1829.h"

#include <xc.h>
#include <stdint.h>

#define _XTAL_FREQ   4000000            // OSCINT 4MHz 

uint8_t  dc = 0;                        // valore per duty
 
// 	FUNZIONI.
// clock a 4MHz
void oscint_4M(void){
    OSCCONbits.IRCF = 0b1101;       // intosc 4MHz
}

// inizializza I/O
void initialize_io(void){
    ANSELA = 0;                     // no analogiche
    ANSELB = 0;                  
    ANSELC = 0;            
    TRISA = 0b11001111;            // RA5:4 out
 // abilita wpu per ingressi
    nWPUEN = 0;                 //OPTION_REGbits.nWPUEN = 0	
	// rilocare P2A su RA5  P2B su RA4
	APFCON1bits.CCP2SEL = 1;
	APFCON1bits.P2BSEL = 1;
}

// setup Timer6 per 1kHz, clock Fosc/4
void initialize_tmr6(void){   
    T6CONbits.T6CKPS = 0b00;    // prescaler 1:1 postscaler 1:1         
	PR6 = 250-1;
}
// Timer2 on & clear flag
void tmr6_on(void){
    TMR6IF = 0;
    TMR6ON = 1;
}

 // setup ECCP/PWM2
void initialize_epwm2(void){
    CCPTMRSbits.C2TSEL = 0b10;     // Select Timer6
    CCP2CONbits.P2M = 0b10;        // half bridge   
	CCP2CONbits.CCP2M =0b1100;     // PxA/PxB active-high 
    CCPR2L = 125;                  // duty 50%
    CCP2CONbits.DC2B = 0b00;       // LSB non usati
    //CCP1AS = 0;
    //PWM1CON = 0;
    //PSTR2CON = 0;     
}

 
/************************* MAIN PROGRAM ****************************/
void main(){
    
   oscint_4M();
   initialize_io();            // Inizializza I/O
   initialize_tmr6();          // setup Timer
   initialize_epwm2();         // setup ECCP 
   tmr6_on();                  // timer on
   
// Loop principale 
  while (1){

    for(dc = 0; dc < 250; dc++){
      CCPR2L = dc;
      __delay_ms(40);
    }
    for(dc = 249; dc > 0; dc--){
      CCPR2L = dc; 
     __delay_ms(40);
	}
  }   
}