/********************************************************************
*--------------------------------------------------------------------
*    Titolo        :  C Enhanced - es16C_3
*                     Half bridge per motore cc.
*                     Variazione velocit/direzione da potenziometro.
*                     Modulo ECCP/PWM2.
*    Data          :  01-05-2011
*    Modificato il :
*    Versione      :  V0.0
*    Ref. Hardware :  16F1829
*    Autore        :  afg
*
*--------------------------------------------------------------------
*********************************************************************
*  Impiego pin :
*  ----------------
*       16F1829 @ 20 pin
*
*                  |\/|
*             Vdd -|1   20|- Vss
*       RA5/CLKIN -|2   19|- RA0/ICSPDAT
*      RA4/CLKOUT -|3   18|- RA1/ICSPCLK
*        RA3/MCLR -|4   17|- RA2
*             RC5 -|5   16|- RCO
*             RC4 -|6   15|- RC1
*             RC3 -|7   14|- RC2
*             RC6 -|8   13|- RB4
*             RC7 -|9   12|- RB5
*             RB7 -|10  11|- RB6
*                  |______|
*
* Impiego pin:
*    Vdd      1: ++
*    RA5      2: Out P2B
*    RA4      3: Out P2B
*    RA3      4: MCLR
*    RC5      5: In
*    RC4      6: In
*    RC3      7: In
*    RC6      8: In
*    RC7      9: In  
*    RB7     10: In
*    RB6     11: In  
*    RB5     12: In  
*    RB4     13: In  
*    RC2     14: In
*    RC1     15: In
*    RC0     16: In
*    RA2     17: In  AN2
*    RA1     18: In  
*    RA0     19: In  
*    Vss     20: --
*
* Note: - INTOSC default @ 500kHz*
********************************************************************/

// intosc, no clockout, no wdt, no pwrt, mclr, bor, no code prot.
// no WRT, no pll, no stack error, bor low, no bor lp, no LVP
#include "C:/Corso_C/MyIncludes/conf1829.h"

#include <xc.h>

#define _XTAL_FREQ   4000000        // OSCINT 4MHz 
 
// 	FUNZIONI.
// clock a 4MHz
void oscint_4M(void){
    OSCCONbits.IRCF = 0b1101;       // intosc 4MHz
}

// inizializza I/O
void initialize_io(void){
    ANSELA = 0b00000100;           // solo AN2-RA2 
    ANSELB = 0;                    // no analogiche
    ANSELC = 0;            
    TRISA = 0b11001111;            // RA5:4 out
	WPUA = 0b00111011;             // no wpu su RA2
 // abilita wpu per ingressi
    nWPUEN = 0;                    // OPTION_REGbits.nWPUEN = 0	
 // rilocare P2A su RA5  P2B su RA4
	APFCON1bits.CCP2SEL = 1;
	APFCON1bits.P2BSEL = 1;
}

// setup Timer4 per 980Hz, clock Fosc/4
void initialize_tmr4(void){   
    T4CONbits.T4CKPS = 0b01;    // prescaler 1:4 postscaler 1:1         
	PR4 = 255-1;
}
// Timer4 on & clear flag
void tmr4_on(void){
    TMR4IF = 0;
    TMR4ON = 1;
}

 // setup ECCP/PWM2
void initialize_epwm2(void){
    CCPTMRSbits.C2TSEL = 0b01;     // Select Timer4
    CCP2CONbits.P2M = 0b10;        // half bridge   
	CCP2CONbits.CCP2M =0b1100;     // PxA/PxB active-high 
    CCPR2L = 125;                  // duty 50%
    CCP2CONbits.DC2B = 0b00;       // LSB non usati
    //CCP1AS = 0;
    //PWM1CON = 0;
    //PSTR2CON = 0;  
    //PWM2CON = 10;
}

/* initializza ADC
   clock FRC, risultato giustificato a sinistra, Vref+ a Vdd
   Vref- a Vss, ingresso da AN2/RA2, no autotrigger */
void initialize_adc(void){   
    ADCON1 = 0x70;              // FRC, just left, Vref=Vdd 
    ADCON0bits.CHS = 0b00010;   // canale AN2
    ADON = 1;                   // ADC on
}

 
/************************* MAIN PROGRAM ****************************/
void main(){
    
   oscint_4M();
   initialize_io();            // Inizializza I/O
   initialize_tmr4();          // setup Timer
   initialize_epwm2();         // setup ECCP 
   initialize_adc();           // setup ADC
   tmr4_on();                  // timer on
   
// Loop principale 
  while (1){

  // conversione AD
   ADCON0bits.GO = 1;                // avvia conversione
     while (ADCON0bits.GO_nDONE) ;   // attesa fine conversione
       
  // aggiorna regidtro duty cycle
        CCPR2L = ADRESH;            // duty = risultato ADC
    } 
}