/******************************************************************
*--------------------------------------------------------------------
*    Titolo        :  C Enhanced - es16C_5
*                     Full bridge per motore cc.
*                     Variazione velocit/direzione da potenziometro.
*                     Auto shutdown e restart da INT.
*                     Modulo ECCP/PWM1.
*    Data          :  01-05-2011
*    Modificato il :
*    Versione      :  V0.0
*    Ref. Hardware :  16F1829
*    Autore        :  afg
*
*--------------------------------------------------------------------
*********************************************************************
*  Impiego pin :
*  ----------------
*       16F1829 @ 20 pin
*
*                  |\/|
*             Vdd -|1   20|- Vss
*       RA5/CLKIN -|2   19|- RA0/ICSPDAT
*      RA4/CLKOUT -|3   18|- RA1/ICSPCLK
*        RA3/MCLR -|4   17|- RA2
*             RC5 -|5   16|- RCO
*             RC4 -|6   15|- RC1
*             RC3 -|7   14|- RC2
*             RC6 -|8   13|- RB4
*             RC7 -|9   12|- RB5
*             RB7 -|10  11|- RB6
*                  |______|
*
* Impiego pin:
*    Vdd      1: ++
*    RA5      2: Out LEDStop
*    RA4      3: Out LEDHalt
*    RA3      4: MCLR
*    RC5      5: Out P1A
*    RC4      6: Out P1B
*    RC3      7: Out P1C
*    RC6      8: In
*    RC7      9: In  
*    RB7     10: In
*    RB6     11: In  
*    RB5     12: In  
*    RB4     13: In  AN10
*    RC2     14: Out P1D
*    RC1     15: Out LEDReverse
*    RC0     16: Out LEDForward
*    RA2     17: In  INT
*    RA1     18: In  
*    RA0     19: In  
*    Vss     20: --
*
* Note: - INTOSC default @ 500kHz*
********************************************************************/

// intosc, no clockout, no wdt, no pwrt, mclr, bor, no code prot.
// no WRT, no pll, no stack error, bor low, no bor lp, no LVP
#include "C:/Corso_C/MyIncludes/conf1829.h"

#include <xc.h>

#define _XTAL_FREQ   4000000        // OSCINT 4MHz 

#define LEDR    LATCbits.LATC0      // led forward
#define LEDF    LATCbits.LATC1      // led reverse
#define LEDS    LATAbits.LATA5      // led stop  rotore fermo
#define LEDH    LATAbits.LATA4      // led halt - shutdown
 
// 	FUNZIONI.
// clock a 4MHz
void oscint_4M(void){
    OSCCONbits.IRCF = 0b1101;       // intosc 4MHz
}

// inizializza I/O
void initialize_io(void){
    ANSELA = 0;                    // no analogiche
    ANSELB = 0b00010000;           // AN10 - RB4
    ANSELC = 0; 
    LATA = 0;                      // preset led spenti
    LATC = 0;                      
    TRISA = 0b00001111;            // RA5:4 out	
    TRISC = 0;                     // RC  out
    WPUB = 0b00101111;             // no wpu su RB4-AN10
 // abilita wpu per ingressi
    nWPUEN = 0;                    // OPTION_REGbits.nWPUEN = 0	
}

// setup Timer2 per 980Hz, clock Fosc/4
void initialize_tmr2(void){   
    T2CONbits.T2CKPS = 0b01;       // prescaler 1:4 postscaler 1:1         
	PR2 = 255-1;
}
// Timer4 on & clear flag
void tmr2_on(void){
    TMR2IF = 0;
    TMR2ON = 1;
}

 // setup ECCP/PWM1
void initialize_epwm1(void){
    //CCPTMRSbits.C1TSEL = 0b00;   // Select Timer2 (defailt)
    CCP1CONbits.P1M = 0b01;        // full bridge forward	
    CCP1CONbits.CCP1M =0b1110;     // P1A/C low P1B/D high
    CCP1CONbits.DC1B = 0b00;       // LSB non usati
    CCP1ASbits.CCP1AS = 0b100;     // vil sul pin INT
    CCP1ASbits.PSS1AC = 0b11;      // P1A/P1C tris-state
    //CCP1ASbits.PSS1AC = 0b01;     // P1A/P1C = 1 driver off
    //CCP1ASbits.PSS1BD = 0b11;     // P1B/P1D tri-state  
    CCP1ASbits.PSS1BD = 0b01;      // P1B/P1D = 1 driver on - freno
    //CCp1ASbits.PSS1BD = 0b00;     // P1B/P1D = 0 driver off
    PWM1CONbits.P1RSEN = 1;        // auto restart
    //PWM1CONbits.PRSEN = 0;        // manual restart
    //PSTR1CON = 0;  
}

/* initializza ADC
   clock FRC, giustificato a sinistra, Vref+ a Vdd
   Vref- a Vss, ingresso da AN10/RB4, no autotrigger */
void initialize_adc(void){   
    ADCON1 = 0x70;              // FRC, just left, Vref=Vdd 
    ADCON0bits.CHS = 0b01010;   // canale AN10
    ADON = 1;                   // ADC on
}

 
/************************* MAIN PROGRAM ****************************/
void main(){
    
   oscint_4M();
   initialize_io();            // Inizializza I/O
   initialize_tmr2();          // setup Timer
   initialize_epwm1();         // setup ECCP 
   initialize_adc();           // setup ADC
   tmr2_on();                  // timer on
   
// Loop principale 
  while (1){

// test per shutdown
   if(CCP1ASE){                      // se shutdown  
     LEDH = 1;                       // LEDH acceso    
   }
   else{                             // altrimenti
     LEDH = 0;                       // LEDH spento
   }

// conversione AD
   ADCON0bits.GO = 1;                // avvia conversione
   while (ADCON0bits.GO_nDONE) ;     // attesa fine conversione

// test per direzione
   if(ADRESH >= 128){                // se forward
     CCPR1L = ((ADRESH - 128)<<1);   // carica duty
     CCP1CONbits.P1M = 0b01;         // direzione forward
	if (ADRESH == 128){              // se 128
	  LEDF=0; LEDS =1;               // acceso solo LEDS 
      } 
	else{                            // altrimenti
        LEDS= 0; LEDF = 1;           // acceso LEDF 
      }	  
   }
   else {                            // se reverse  
     CCPR1L = ((127 - ADRESH)<<1);   // carica duty
     CCP1CONbits.P1M = 0b11;         // direzione reverse
	if(ADRESH == 127){               // se 127 
	  LEDR=0; LEDS =1;	             // acceso solo LEDS   
      }
	else{                            // altrimenti
        LEDS= 0; LEDR=1;             // acceso LEDR 	 
      }	  
   }	  
  } 
}
